<?php declare(strict_types=1);
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#


namespace Platiti\UniAdapter\Service;

use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\AsynchronousPaymentHandlerInterface;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Exception;
use Platiti\PtiUniNs\UniModul\UniErr;

class UniPayment implements AsynchronousPaymentHandlerInterface
{
	private string $uniModulName;
	private UniPaymentAdapter $adapter;

	public function __construct(string $uniModulName, UniPaymentAdapter $adapter) {
		UniErr::run(null, function() use ($uniModulName, $adapter) {
			$this->adapter = $adapter;
			$this->uniModulName = $uniModulName;
		});
	}


	public function pay(AsyncPaymentTransactionStruct $transaction, RequestDataBag $dataBag, SalesChannelContext $salesChannelContext): RedirectResponse
	{
		return UniErr::run(null, function() use ($transaction, $salesChannelContext) {
			$resp = $this->adapter->pay($this->uniModulName, $transaction, $salesChannelContext);
			if ($resp instanceof RedirectResponse) {
				return $resp;
			} else {
				echo $resp->getContent();
				exit();
			}
		});
	}


	public function finalize(AsyncPaymentTransactionStruct $transaction, Request $request, SalesChannelContext $salesChannelContext): void
	{
		UniErr::run(null, function() {
			throw new Exception('Not implemented this way');
		});
	}

}
