<?php declare(strict_types=1);
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#


namespace Platiti\UniAdapter\Service;

use Symfony\Component\HttpFoundation\RequestStack;
use Shopware\Core\System\Locale\LanguageLocaleCodeProvider;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Psr\Container\ContainerInterface;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStates;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\System\StateMachine\StateMachineEntity;
use Shopware\Core\Checkout\Payment\PaymentException;
use \Exception;

use Platiti\PtiUniNs\UniModul\UniModulFactory;
use Platiti\PtiUniNs\UniModul\UniModulConfig;
use Platiti\PtiUniNs\UniModul\OrderStatus;
use Platiti\PtiUniNs\UniModul\UniModul;
use Platiti\PtiUniNs\UniModul\ConfigInfo;
use Platiti\PtiUniNs\UniModul\ConfigSetting;
use Platiti\PtiUniNs\UniModul\CustomerData;
use Platiti\PtiUniNs\UniModul\OrderToPayInfo;
use Platiti\PtiUniNs\UniModul\OrderReplyStatus;
use function Platiti\PtiUniNs\UniModul\create_initialize_object;


if (!defined('E_UNIERR_DEFAULT')) {
	define('E_UNIERR_DEFAULT', E_ALL & ~E_STRICT & ~E_USER_DEPRECATED);
}

require_once(__DIR__ . '/../UniModul/UniModul.php');


class UniPaymentAdapter
{
	protected $adapterName = 'Shopware';
	/** @var UniModul */
	private $uniModul;
	/** @var OrderTransactionStateHandler*/
	private $transactionStateHandler;
	/** @var ConfigInfo */
	protected $configInfo;
	/** @var ConfigSetting */
	protected $configSetting;

	/** @var ContainerInterface */
	private $container;
	/** @var RequestStack */
	private $requestStack;
	/** @var LanguageLocaleCodeProvider */
	private $languageLocaleProvider;
	/** @var SystemConfigService */
	private $systemConfigService;
	/** @var OrderTransactionStateHandler */
	private $transactionStateHandle; // @phpstan-ignore-line
	private EntityRepository $stateMachineStateRepository;

	public function __construct(
			ContainerInterface $container,
			RequestStack $requestStack,
			LanguageLocaleCodeProvider $languageLocaleProvider,
			SystemConfigService $systemConfigService,
			OrderTransactionStateHandler $transactionStateHandler,
			EntityRepository $stateMachineStateRepository
			)
	{
		$this->container = $container;
		$this->requestStack = $requestStack;
		$this->languageLocaleProvider = $languageLocaleProvider;
		$this->systemConfigService = $systemConfigService;
		$this->transactionStateHandler = $transactionStateHandler;
		$this->stateMachineStateRepository = $stateMachineStateRepository;
	}

	public function pay(string $uniModulName, $transaction, $salesChannelContext) : Response {
		$this->makeUniModul($uniModulName, $salesChannelContext);
		$orderToPayInfo = $this->getOrderToPayInfo($transaction, $salesChannelContext);
		$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);
//
//
//			if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
//				$frontend_redir = $redirectAction->redirectUrl==null && $redirectAction->redirectForm==null;
//				$html='';
//				$this->procesReplyStatus($redirectAction->orderReplyStatus, $frontend_redir, $html);
//				vRequest::setVar ('html', $html);
//			}

		if ($redirectAction->redirectUrl != null) {
			return new RedirectResponse($redirectAction->redirectUrl);
		} else if ($redirectAction->redirectForm != null) {
			return new Response($this->uniModul->formRedirect($redirectAction->redirectForm));
		} else {
			throw new Exception("Payment redirect failed: " . $redirectAction->orderReplyStatus->resultText);
		}

	}

	public function receiveReply(Request $request, SalesChannelContext $salesChannelContext) :string {
		$this->makeUniModul($request->get('uniModul'), $salesChannelContext);
		/** @var null|OrderReplyStatus $orderReplyStatus */
		$orderReplyStatus = $this->uniModul->gatewayReceiveReply('en' /*TODO*/);
		$transaction = $this->getTransactionById($orderReplyStatus->shopPairingInfo, $salesChannelContext);
		return $this->processReplyStatus($orderReplyStatus, $transaction, $salesChannelContext);
	}


	protected function getOrderToPayInfo($transaction, $salesChannelContext) {

		$order = $transaction->getOrder();

		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->shopOrderNumber = $order->getOrderNumber();
		$orderToPayInfo->shopPairingInfo = $transaction->getOrderTransaction()->getId();
		$orderToPayInfo->amount = $order->getAmountTotal();
		$orderToPayInfo->currency = $order->getCurrency()->getIsoCode();

		$language = $orderToPayInfo->language = $order->getLanguage();
		$localeCode = $this->languageLocaleProvider->getLocaleForLanguageId($language->getId());
		$orderToPayInfo->language = substr($localeCode, 0, 2);

		$orderToPayInfo->replyUrl = $this->generateUrl('frontend.uniadapter.reply', ['uniModul' => $this->uniModul->name], UrlGeneratorInterface::ABSOLUTE_URL);

		// $orderToPayInfo->uniModulDirUrl = null; // neumime se dostat zvenku na slozku UniModul
		$currentRequest = $this->requestStack->getCurrentRequest();
		$urlPublic = $currentRequest->getSchemeAndHttpHost() . $currentRequest->getBaseUrl();
		$orderToPayInfo->uniModulDirUrl = $urlPublic . "/bundles/PlatitiUniAdapter/UniModul/";

		$customerData = new CustomerData();
		$customer = $order->getOrderCustomer();
		$billingAddress = $order->getBillingAddress();
		$customerData->first_name = $customer->getFirstName();
		$customerData->last_name = $customer->getLastName();
		$customerData->email = $customer->getEmail();
		$customerData->street = $billingAddress->getStreet();
		$customerData->city = $billingAddress->getCity();
		$customerData->post_code = $billingAddress->getZipcode();
		$customerData->state = $billingAddress->getCountryState() != null ? $billingAddress->getCountryState()->getName() : null;
		$customerData->country = $billingAddress->getCountry()->getIso();
		$customerData->phone = $billingAddress->getPhoneNumber();
		$customerData->identifier = $customer->getCustomerId();
		$orderToPayInfo->customerData = $customerData;

		return $orderToPayInfo;
	}

	protected function getTransactionById(string $transactionId, SalesChannelContext $salesChannelContext) {
		$criteria = new Criteria([$transactionId]);
		$criteria->addAssociation('order.transactions');
		$criteria->addAssociation('order.currency');

		$transactionRepository = $this->container->get('order_transaction.repository');
		/** @var null|OrderTransactionEntity $transaction */
		$transaction = $transactionRepository->search($criteria, $salesChannelContext->getContext())->first();

		if (null === $transaction) {
			throw new NotFoundHttpException('TransactionId not found');
		}
		return $transaction;
	}

	protected function processReplyStatus($orderReplyStatus, OrderTransactionEntity $transaction, SalesChannelContext $salesChannelContext) : string {

		$context = $salesChannelContext->getContext();

		$transactionStateMachineStateId = $transaction->getStateId();
		$criteria = new Criteria([$transactionStateMachineStateId]);

		/** @var StateMachineEntity|null $stateMachineState */
		$stateMachineState = $this->stateMachineStateRepository->search(
			$criteria,
			$salesChannelContext->getContext()
		)->get($transactionStateMachineStateId);

		$currentMachState = $stateMachineState->getTechnicalName();
		if ($orderReplyStatus->orderStatus == OrderStatus::$successful) {
			if (in_array($currentMachState, [OrderTransactionStates::STATE_OPEN, OrderTransactionStates::STATE_FAILED])) {
				$this->logTransactionStatusUpdate($transaction, $currentMachState, 'PAID', true);
				$this->transactionStateHandler->paid($transaction->getId(), $context);
			} else {
				$this->logTransactionStatusUpdate($transaction, $currentMachState, 'PAID', false);
			}
			$guiOk = true;
		} else if ($orderReplyStatus->orderStatus == OrderStatus::$failedFinal || $orderReplyStatus->orderStatus == OrderStatus::$failedRetriable) {
			if (in_array($currentMachState, [OrderTransactionStates::STATE_OPEN])) {
				$this->logTransactionStatusUpdate($transaction, $currentMachState, 'FAILED', true);
				$this->transactionStateHandler->fail($transaction->getId(), $context);
			} else {
				$this->logTransactionStatusUpdate($transaction, $currentMachState, 'FAILED', false);
			}
			$guiOk = false;
		} else if ($orderReplyStatus->orderStatus == OrderStatus::$pending) {
			if (in_array($currentMachState, [OrderTransactionStates::STATE_FAILED])) {
				$this->logTransactionStatusUpdate($transaction, $currentMachState, 'REOPEN', true);
				$this->transactionStateHandler->reopen($transaction->getId(), $context);
			} else {
				$this->logTransactionStatusUpdate($transaction, $currentMachState, 'REOPEN', false);
			}
			$guiOk = true;
		} else if ($orderReplyStatus->orderStatus == OrderStatus::$invalidReply) {
			$this->logTransactionStatusUpdate($transaction, $currentMachState, 'NO ACTION FOR INVALID REPLY', false);
			$guiOk = false;
		} else {
			$this->logTransactionStatusUpdate($transaction, $currentMachState, 'UNEXPECTED STATUS', false);
			$guiOk = false;
		}
		if ($guiOk) {
			return $this->generateUrl('frontend.checkout.finish.page', ['orderId' => $transaction->getOrder()->getId()]);
		} else {
			return $this->generateUrl('frontend.account.edit-order.page', [
				'orderId' => $transaction->getOrder()->getId(),
				'error-code' => PaymentException::asyncFinalizeInterrupted('', '')->getErrorCode()
			]);
		}
	}

	private function logTransactionStatusUpdate(OrderTransactionEntity $transaction, string $currentMachState, string $action, bool $done) {
		$this->uniModul->logger->writeLog("ADAPTER: ".($done ? "Updating" : "Not updating") . " shop order number: ".$transaction->getOrder()->getOrderNumber().", transactionId:".$transaction->getId().", current state: " . $currentMachState . " as $action");
	}

	private function makeUniModul(string $uniModulName, SalesChannelContext $salesChannelContext) {
		$uniFact = new UniModulFactory();
		$configInfo = $uniFact->getConfigInfo($uniModulName, /*TODO*/ 'en', $this->adapterName);
		$this->configInfo = $configInfo;

		$configSetting = $this->getConfigData($this->configInfo, $uniModulName, $salesChannelContext);
		$this->configSetting = $configSetting;
		$this->uniModul = $uniFact->createUniModul($uniModulName, $configSetting);
	}

	protected function getConfigData($configInfo, $uniModulName, SalesChannelContext $salesChannelContext) {
		$configData = array();
		foreach ($configInfo->configFields as $configField) {
			$configData[$configField->name] = $this->systemConfigService->get('Platiti'.$uniModulName.".config.".$configField->name, $salesChannelContext->getSalesChannelId());
		}


		$uniModulConfig = new UniModulConfig();
		$dbUrl = parse_url($_ENV['DATABASE_URL']);
		$uniModulConfig->mysql_server       = urldecode($dbUrl['host']);
		$uniModulConfig->mysql_dbname       = urldecode(basename($dbUrl['path']));
		$uniModulConfig->mysql_login        = urldecode($dbUrl['user']);
		$uniModulConfig->mysql_password     = urldecode($dbUrl['pass'] ?? '');

		$currentRequest = $this->requestStack->getCurrentRequest();
		$urlPublic = $currentRequest->getSchemeAndHttpHost() . $currentRequest->getBaseUrl();
		$uniModulConfig->uniModulDirUrl = $urlPublic . "/bundles/PlatitiUniAdapter/UniModul/";
		$uniModulConfig->adapterName = $this->adapterName;

		$cfgs = create_initialize_object('ConfigSetting', array('configData'=>$configData, 'uniModulConfig'=>$uniModulConfig));
		return $cfgs;
	}

	protected function generateUrl(string $route, array $parameters = [], int $referenceType = UrlGeneratorInterface::ABSOLUTE_PATH): string
	{
		return $this->container->get('router')->generate($route, $parameters, $referenceType);
	}


}

